@@ -30,7 +30,6 @@
 #include "hildon-home-area.h"
 #include "hildon-desktop-marshalers.h"
 
-#include <gtk/gtkfixed.h>
 #include <gtk/gtkicontheme.h>
 
 #include <X11/extensions/Xrender.h>
@@ -42,14 +41,16 @@
 #endif
 
 #define APPLET_RESIZE_HANDLE_ICON   "qgn_home_layoutmode_resize"
-#define APPLET_RESIZE_HANDLE_WIDTH  40
-#define APPLET_RESIZE_HANDLE_HEIGHT 40
+#define APPLET_RESIZE_HANDLE_WIDTH                      36
+#define APPLET_RESIZE_HANDLE_HEIGHT                     36
+#define APPLET_VERTICAL_RESIZE_HANDLE_WIDTH             50
+#define APPLET_VERTICAL_RESIZE_HANDLE_HEIGHT            36
+#define APPLET_HORIZONTAL_RESIZE_HANDLE_WIDTH           36
+#define APPLET_HORIZONTAL_RESIZE_HANDLE_HEIGHT          50
+
 #define APPLET_CLOSE_BUTTON_ICON    "qgn_home_layoutmode_close"
 #define APPLET_CLOSE_BUTTON_WIDTH   26
 #define APPLET_CLOSE_BUTTON_HEIGHT  26
-#define APPLET_DRAG_HANDLE_ICON     "qgn_list_presence_invisible"
-#define APPLET_DRAG_HANDLE_WIDTH    26
-#define APPLET_DRAG_HANDLE_HEIGHT   26
 
 #define GRID_SIZE                   10
 
@@ -60,7 +61,7 @@
   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), HILDON_DESKTOP_TYPE_HOME_ITEM, HildonDesktopHomeItemPriv));
 
 
-typedef enum 
+typedef enum
 {
   HILDON_DESKTOP_HOME_ITEM_STATE_NORMAL = 0,
   HILDON_DESKTOP_HOME_ITEM_STATE_RESIZING,
@@ -87,8 +88,7 @@ typedef struct HildonDesktopHomeItemPriv_
   GdkWindow    *close_button_window;
   GdkPixbuf    *resize_handle;
   GdkWindow    *resize_handle_window;
-  GdkPixbuf    *drag_handle;
-  GdkWindow    *drag_handle_window;
+  gint          resize_handle_width, resize_handle_height;
 
   GdkWindow    *event_window;
 
@@ -139,7 +139,9 @@ static GdkWindow *
 hildon_desktop_home_item_create_icon_window (HildonDesktopHomeItem *applet,
                                              GdkPixbuf *icon,
                                              gint x,
-                                             gint y);
+                                             gint y,
+                                             gint width,
+                                             gint height);
 
 static gboolean
 hildon_desktop_home_item_button_press_event (GtkWidget *applet,
@@ -206,7 +208,7 @@ hildon_desktop_home_item_resize_type_get_type (void)
   static GType etype = 0;
   if (etype == 0)
     {
-    
+
       static const GEnumValue values[] = {
           { HILDON_DESKTOP_HOME_ITEM_RESIZE_NONE,
             "HILDON_DESKTOP_HOME_ITEM_RESIZE_NONE",
@@ -447,33 +449,6 @@ hildon_desktop_home_item_class_init (HildonDesktopHomeItemClass * applet_class)
       error = NULL;
     }
 
-  applet_class->resize_handle =
-      gtk_icon_theme_load_icon (icon_theme,
-                                APPLET_RESIZE_HANDLE_ICON,
-                                APPLET_RESIZE_HANDLE_WIDTH,
-                                GTK_ICON_LOOKUP_NO_SVG,
-                                &error);
-  if (error)
-    {
-      g_warning ("Could not load resize handle icon: %s", error->message);
-      applet_class->resize_handle = NULL;
-      g_error_free (error);
-      error = NULL;
-    }
-
-  applet_class->drag_handle =
-      gtk_icon_theme_load_icon (icon_theme,
-                                APPLET_DRAG_HANDLE_ICON,
-                                APPLET_DRAG_HANDLE_WIDTH,
-                                GTK_ICON_LOOKUP_NO_SVG,
-                                &error);
-  if (error)
-    {
-      g_warning ("Could not load drag handle icon: %s", error->message);
-      applet_class->drag_handle = NULL;
-      g_error_free (error);
-      error = NULL;
-    }
 }
 
 static void
@@ -496,18 +471,6 @@ hildon_desktop_home_item_init (HildonDesktopHomeItem * self)
       priv->close_button = klass->close_button;
     }
 
-  if (klass->resize_handle)
-    {
-      g_object_ref (klass->resize_handle);
-      priv->resize_handle = klass->resize_handle;
-    }
-
-  if (klass->drag_handle)
-    {
-      g_object_ref (klass->drag_handle);
-      priv->drag_handle = klass->drag_handle;
-    }
-
   gtk_widget_add_events (GTK_WIDGET (self), GDK_VISIBILITY_NOTIFY_MASK);
 
 }
@@ -667,43 +630,48 @@ hildon_desktop_home_item_realize (GtkWidget *widget)
 
       gdk_window_set_user_data (widget->window, widget);
 
-#if 0
-      gtk_style_set_background (widget->style,
-                                widget->window,
-                                GTK_STATE_NORMAL);
-#endif
-
-
-      if (0)
-/*      if (!priv->layout_mode_sucks)*/
-        {
-          /* When layout mode is cool, we have an event window
-           * on top of everything to catch the clicks */
-          attributes.wclass = GDK_INPUT_ONLY;
-          attributes_mask = GDK_WA_X | GDK_WA_Y;
-          priv->event_window =  gdk_window_new (widget->window,
-                                                &attributes,
-                                                attributes_mask);
-          gdk_window_set_user_data (priv->event_window, widget);
-        }
-
       if (priv->layout_mode_sucks)
         {
-#if 0
-          priv->drag_handle_window =
-              hildon_desktop_home_item_create_icon_window (HILDON_DESKTOP_HOME_ITEM (widget),
-                                                     priv->drag_handle,
-                                                     0,
-                                                     0);
-#endif
+          if (priv->resize_handle)
+            {
+              priv->resize_handle_width  =
+                  gdk_pixbuf_get_width  (priv->resize_handle);
+              priv->resize_handle_height =
+                  gdk_pixbuf_get_height (priv->resize_handle);
+            }
+          else
+            {
+              switch (priv->resize_type)
+                {
+                  case HILDON_DESKTOP_HOME_ITEM_RESIZE_HORIZONTAL:
+                      priv->resize_handle_width =
+                          APPLET_HORIZONTAL_RESIZE_HANDLE_WIDTH;
+                      priv->resize_handle_height =
+                          APPLET_HORIZONTAL_RESIZE_HANDLE_HEIGHT;
+                      break;
+                  case HILDON_DESKTOP_HOME_ITEM_RESIZE_VERTICAL:
+                      priv->resize_handle_width =
+                          APPLET_VERTICAL_RESIZE_HANDLE_WIDTH;
+                      priv->resize_handle_height =
+                          APPLET_VERTICAL_RESIZE_HANDLE_HEIGHT;
+                      break;
+                  default:
+                      priv->resize_handle_width =
+                          APPLET_RESIZE_HANDLE_WIDTH;
+                      priv->resize_handle_height = APPLET_RESIZE_HANDLE_HEIGHT;
+                      break;
+                }
+            }
 
-          if (priv->resize_type != HILDON_DESKTOP_HOME_ITEM_RESIZE_NONE)
-            priv->resize_handle_window =
-                hildon_desktop_home_item_create_icon_window
-                    (HILDON_DESKTOP_HOME_ITEM (widget),
-                     priv->resize_handle,
-                     widget->allocation.width - APPLET_RESIZE_HANDLE_WIDTH,
-                     widget->allocation.height - APPLET_RESIZE_HANDLE_HEIGHT);
+          priv->resize_handle_window =
+              hildon_desktop_home_item_create_icon_window
+              (HILDON_DESKTOP_HOME_ITEM (widget),
+               priv->resize_handle,
+               widget->allocation.width  - priv->resize_handle_width,
+               widget->allocation.height - priv->resize_handle_height,
+               priv->resize_handle_width,
+               priv->resize_handle_height
+              );
 
           gdk_window_set_events (widget->window,
                                  gdk_window_get_events(widget->window) |
@@ -763,11 +731,6 @@ hildon_desktop_home_item_map (GtkWidget *widget)
    * on top */
   if (GDK_IS_WINDOW (priv->event_window))
     gdk_window_show (priv->event_window);
-
-  if (priv->drag_handle_window)
-    gdk_window_raise (priv->drag_handle_window);
-
-
 }
 
 static void
@@ -955,9 +918,9 @@ hildon_desktop_home_item_size_allocate (GtkWidget *widget,
       if (GDK_IS_WINDOW (priv->resize_handle_window))
         gdk_window_move (priv->resize_handle_window,
                          allocation->width
-                         - APPLET_RESIZE_HANDLE_WIDTH,
+                         - priv->resize_handle_width,
                          allocation->height
-                         - APPLET_RESIZE_HANDLE_HEIGHT);
+                         - priv->resize_handle_height);
 
       if (GDK_IS_WINDOW (widget->window))
         gdk_window_move_resize (widget->window,
@@ -1006,35 +969,28 @@ static GdkWindow *
 hildon_desktop_home_item_create_icon_window (HildonDesktopHomeItem *applet,
                                              GdkPixbuf *icon,
                                              gint x,
-                                             gint y)
+                                             gint y,
+                                             gint width,
+                                             gint height)
 {
   HildonDesktopHomeItemPriv      *priv;
   GdkWindowAttr              attributes;
   gint                       attributes_mask;
-  GdkBitmap                 *bitmask = NULL;
-  GdkPixmap                 *pixmap = NULL;
   GdkWindow                 *window;
   GtkWidget                 *w;
 
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (applet);
   w = GTK_WIDGET (applet);
 
-  attributes.width = gdk_pixbuf_get_width (icon);
-  attributes.height = gdk_pixbuf_get_height (icon);
+  attributes.width  = width;
+  attributes.height = height;
   attributes.window_type = GDK_WINDOW_CHILD;
   attributes.event_mask = 0;
   attributes.visual = gtk_widget_get_visual (w);
   attributes.colormap = gtk_widget_get_colormap (w);
-  attributes.wclass = GDK_INPUT_OUTPUT;
+  attributes.wclass = icon?GDK_INPUT_OUTPUT:GDK_INPUT_ONLY;
   attributes_mask = GDK_WA_VISUAL | GDK_WA_COLORMAP;
 
-  gdk_pixbuf_render_pixmap_and_mask_for_colormap (icon,
-                                                  attributes.colormap,
-                                                  &pixmap,
-                                                  &bitmask,
-                                                  127);
-
-
   window = gdk_window_new (w->window,
                            &attributes,
                            attributes_mask);
@@ -1046,20 +1002,33 @@ hildon_desktop_home_item_create_icon_window (HildonDesktopHomeItem *applet,
 
   gdk_window_set_user_data (window, w);
   gdk_window_move (window, x, y);
-
-  if (pixmap)
+  if (icon)
     {
-      gdk_window_set_back_pixmap (window, pixmap, FALSE);
-      g_object_unref (pixmap);
-    }
+      GdkBitmap                 *bitmask = NULL;
+      GdkPixmap                 *pixmap = NULL;
 
-  if (bitmask)
-    {
-      gdk_window_shape_combine_mask (window,
-                                     bitmask,
-                                     0,
-                                     0);
-      g_object_unref (bitmask);
+      gdk_pixbuf_render_pixmap_and_mask_for_colormap (icon,
+                                                      attributes.colormap,
+                                                      &pixmap,
+                                                      &bitmask,
+                                                      127);
+
+
+
+      if (pixmap)
+        {
+          gdk_window_set_back_pixmap (window, pixmap, FALSE);
+          g_object_unref (pixmap);
+        }
+
+      if (bitmask)
+        {
+          gdk_window_shape_combine_mask (window,
+                                         bitmask,
+                                         0,
+                                         0);
+          g_object_unref (bitmask);
+        }
     }
 
   gdk_window_show (window);
@@ -1089,8 +1058,6 @@ window_event_filter (GdkXEvent *xevent,
           gdk_window_raise (priv->close_button_window);
       if (priv->resize_handle_window)
         gdk_window_raise (priv->resize_handle_window);
-      if (priv->drag_handle_window)
-        gdk_window_raise (priv->drag_handle_window);
 
       if (!priv->layout_mode_sucks && GTK_IS_WIDGET (GTK_BIN (applet)->child))
         {
@@ -1136,21 +1103,25 @@ hildon_desktop_home_item_layout_mode_start (HildonDesktopHomeItem *applet)
 
   if (priv->close_button)
     {
-      priv->close_button_window = hildon_desktop_home_item_create_icon_window 
+      priv->close_button_window = hildon_desktop_home_item_create_icon_window
           (applet,
            priv->close_button,
            HILDON_MARGIN_DEFAULT,
-           HILDON_MARGIN_DEFAULT);
+           HILDON_MARGIN_DEFAULT,
+           gdk_pixbuf_get_width (priv->close_button),
+           gdk_pixbuf_get_height (priv->close_button));
     }
 
   if (priv->resize_handle &&
       priv->resize_type != HILDON_DESKTOP_HOME_ITEM_RESIZE_NONE)
     {
-      priv->resize_handle_window = hildon_desktop_home_item_create_icon_window 
+      priv->resize_handle_window = hildon_desktop_home_item_create_icon_window
           (applet,
            priv->resize_handle,
            widget->allocation.width - APPLET_RESIZE_HANDLE_WIDTH,
-           widget->allocation.height - APPLET_RESIZE_HANDLE_HEIGHT);
+           widget->allocation.height - APPLET_RESIZE_HANDLE_HEIGHT,
+           gdk_pixbuf_get_width (priv->resize_handle),
+           gdk_pixbuf_get_height (priv->resize_handle));
     }
 
   if (GTK_IS_WIDGET (GTK_BIN (applet)->child))
@@ -1246,7 +1217,7 @@ hildon_desktop_home_item_drag_update (HildonDesktopHomeItem *applet)
   HildonDesktopHomeItemPriv *priv;
   gint                  x_applet, y_applet;
   GdkModifierType       mod;
-  GtkWidget            *fixed;
+  GtkWidget            *area;
   GtkWidget            *widget;
   gboolean              used_to_overlap;
 
@@ -1257,17 +1228,17 @@ hildon_desktop_home_item_drag_update (HildonDesktopHomeItem *applet)
 
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (applet);
 
-  fixed = widget->parent;
+  area = widget->parent;
 
-  gdk_window_get_pointer (fixed->window, &x_applet, &y_applet, &mod);
+  gdk_window_get_pointer (area->window, &x_applet, &y_applet, &mod);
 
   if (priv->state == HILDON_DESKTOP_HOME_ITEM_STATE_MOVING)
     {
       /* The Fixed has no window, thus the coordinates are relative to the
        * home's main window */
 
-      x_applet -= fixed->allocation.x;
-      y_applet -= fixed->allocation.y;
+      x_applet -= area->allocation.x;
+      y_applet -= area->allocation.y;
 
       x_applet -= priv->x_offset;
       y_applet -= priv->y_offset;
@@ -1278,20 +1249,24 @@ hildon_desktop_home_item_drag_update (HildonDesktopHomeItem *applet)
       if (y_applet < 0)
         y_applet = 0;
 
-      if (x_applet + widget->allocation.width > fixed->allocation.width)
-        x_applet =  fixed->allocation.width -
+      if (x_applet + widget->allocation.width > area->allocation.width)
+        x_applet =  area->allocation.width -
             widget->allocation.width;
 
-      if (y_applet + widget->allocation.height > fixed->allocation.height)
-        y_applet =  fixed->allocation.height - widget->allocation.height;
+      if (y_applet + widget->allocation.height > area->allocation.height)
+        y_applet =  area->allocation.height - widget->allocation.height;
 
-      if (x_applet != widget->allocation.x - fixed->allocation.x)
-        priv->delta_x = x_applet - widget->allocation.x + fixed->allocation.x;
+      if (x_applet != widget->allocation.x - area->allocation.x)
+        priv->delta_x = x_applet - widget->allocation.x + area->allocation.x;
 
-      if (y_applet != widget->allocation.y - fixed->allocation.y)
-        priv->delta_y = y_applet - widget->allocation.y + fixed->allocation.y;
+      if (y_applet != widget->allocation.y - area->allocation.y)
+        priv->delta_y = y_applet - widget->allocation.y + area->allocation.y;
+
+      hildon_home_area_move (HILDON_HOME_AREA (area),
+                             widget,
+                             x_applet,
+                             y_applet);
 
-      gtk_fixed_move (GTK_FIXED (fixed), widget, x_applet, y_applet);
     }
   else /* Resizing */
     {
@@ -1316,13 +1291,13 @@ hildon_desktop_home_item_drag_update (HildonDesktopHomeItem *applet)
         height = priv->minimum_height;
 
       if (widget->allocation.x + width >
-          fixed->allocation.x + fixed->allocation.width)
-        width = fixed->allocation.x + fixed->allocation.width
+          area->allocation.x + area->allocation.width)
+        width = area->allocation.x + area->allocation.width
                 - widget->allocation.x;
 
       if (widget->allocation.y + height >
-          fixed->allocation.y + fixed->allocation.height)
-        height = fixed->allocation.y + fixed->allocation.height
+          area->allocation.y + area->allocation.height)
+        height = area->allocation.y + area->allocation.height
                  - widget->allocation.y;
 
       if (width != widget->allocation.width)
@@ -1381,7 +1356,7 @@ hildon_desktop_home_item_snap_to_grid (HildonDesktopHomeItem *applet)
   widget = GTK_WIDGET (applet);
   parent = widget->parent;
 
-  if (!GTK_IS_FIXED (parent))
+  if (!HILDON_IS_HOME_AREA (parent))
     return;
 
   x = widget->allocation.x - parent->allocation.x;
@@ -1405,7 +1380,9 @@ hildon_desktop_home_item_snap_to_grid (HildonDesktopHomeItem *applet)
                        parent->allocation.height))
                 y_grid += GRID_SIZE;
 
-              gtk_fixed_move (GTK_FIXED (parent), widget, x_grid, y_grid);
+             hildon_home_area_move (HILDON_HOME_AREA (parent),
+                                    widget,
+                                    x_grid, y_grid);
             }
 
           break;
@@ -1592,7 +1569,7 @@ hildon_desktop_home_item_key_press_event (GtkWidget *widget,
 
   if (GTK_WIDGET_CLASS (parent_class)->key_press_event)
     return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
-    
+
   return FALSE;
 }
 
@@ -1608,7 +1585,7 @@ hildon_desktop_home_item_key_release_event (GtkWidget *widget,
 
   if (GTK_WIDGET_CLASS (parent_class)->key_release_event)
     return GTK_WIDGET_CLASS (parent_class)->key_release_event (widget, event);
-    
+
   return FALSE;
 }
 
@@ -1634,7 +1611,7 @@ hildon_desktop_home_item_get_layout_mode (HildonDesktopHomeItem *applet)
   g_return_val_if_fail (applet, FALSE);
 
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (applet);
-  
+
   return priv->layout_mode;
 }
 
@@ -1646,7 +1623,7 @@ hildon_desktop_home_item_set_layout_mode (HildonDesktopHomeItem *applet,
   g_return_if_fail (applet);
 
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (applet);
-  
+
   if (priv->layout_mode != layout_mode)
     {
       g_object_notify (G_OBJECT (applet), "layout-mode");
@@ -1665,9 +1642,9 @@ hildon_desktop_home_item_get_resize_type (HildonDesktopHomeItem *applet)
 {
   HildonDesktopHomeItemPriv      *priv;
   g_return_val_if_fail (applet, FALSE);
-  
+
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (applet);
-  
+
   return priv->resize_type;
 }
 
@@ -1675,15 +1652,61 @@ void
 hildon_desktop_home_item_set_resize_type (HildonDesktopHomeItem *applet,
                                           HildonDesktopHomeItemResizeType resize_type)
 {
-  HildonDesktopHomeItemPriv      *priv;
+  HildonDesktopHomeItemPriv    *priv;
+  GtkWidget                    *widget;
   g_return_if_fail (applet);
 
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (applet);
-          
+  widget = GTK_WIDGET (applet);
+
   if (priv->resize_type != resize_type)
     {
       g_object_notify (G_OBJECT (applet), "resize-type");
       priv->resize_type = resize_type;
+
+      if (GTK_WIDGET_REALIZED (widget) &&
+          priv->resize_type != HILDON_DESKTOP_HOME_ITEM_RESIZE_NONE)
+        {
+          gint icon_width, icon_height;
+
+          if (GDK_IS_WINDOW (priv->resize_handle_window))
+              gdk_window_destroy (priv->resize_handle_window);
+
+          if (priv->resize_handle)
+            {
+              icon_width  = gdk_pixbuf_get_width  (priv->resize_handle);
+              icon_height = gdk_pixbuf_get_height (priv->resize_handle);
+            }
+          else
+            {
+              switch (priv->resize_type)
+                {
+                  case HILDON_DESKTOP_HOME_ITEM_RESIZE_HORIZONTAL:
+                      icon_width  = APPLET_HORIZONTAL_RESIZE_HANDLE_WIDTH;
+                      icon_height = APPLET_HORIZONTAL_RESIZE_HANDLE_HEIGHT;
+                      break;
+                  case HILDON_DESKTOP_HOME_ITEM_RESIZE_VERTICAL:
+                      icon_width  = APPLET_VERTICAL_RESIZE_HANDLE_WIDTH;
+                      icon_height = APPLET_VERTICAL_RESIZE_HANDLE_HEIGHT;
+                      break;
+                  default:
+                      icon_width  = APPLET_RESIZE_HANDLE_WIDTH;
+                      icon_height = APPLET_RESIZE_HANDLE_HEIGHT;
+                      break;
+                }
+            }
+
+          priv->resize_handle_window =
+              hildon_desktop_home_item_create_icon_window
+              (applet,
+               priv->resize_handle,
+               widget->allocation.width  - icon_width,
+               widget->allocation.height - icon_height,
+               icon_width,
+               icon_height
+              );
+        }
+
     }
 }
 
@@ -1694,9 +1717,9 @@ hildon_desktop_home_item_get_settings_menu_item (HildonDesktopHomeItem *applet)
   GtkWindow                 *window;
   GtkWidget                 *item = NULL;
   g_return_val_if_fail (applet, NULL);
-  
+
   priv = HILDON_DESKTOP_HOME_ITEM_GET_PRIVATE (applet);
-  
+
   window = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (applet)));
 
   g_signal_emit_by_name (applet, "settings", window, &item);
