@@ -29,6 +29,14 @@
 #include <gdk/gdkx.h>
 #include <gdk/gdkwindow.h>
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_XTEST
+#include <X11/extensions/XTest.h>
+#endif
+
 #define HILDON_DESKTOP_POPUP_WINDOW_GET_PRIVATE(object) \
         (G_TYPE_INSTANCE_GET_PRIVATE ((object), HILDON_DESKTOP_TYPE_POPUP_WINDOW, HildonDesktopPopupWindowPrivate))
 
@@ -89,9 +97,10 @@ static gboolean hildon_desktop_popup_window_composited_button_release (GtkWidget
 
 static gboolean popup_grab_on_window (GdkWindow *window, guint32 activate_time, gboolean grab_keyboard);
 
-static gboolean
-hildon_desktop_popup_window_button_release_event (GtkWidget *widget, GdkEventButton *event);
-
+static gboolean hildon_desktop_popup_window_button_release_event (GtkWidget *widget, GdkEventButton *event);
+#ifdef HAVE_XTEST
+static void hildon_desktop_popup_menu_fake_button_event (GdkEventButton *event, gboolean press);
+#endif
 struct _HildonDesktopPopupWindowPrivate 
 {
   GtkWidget	   	          **extra_panes;
@@ -631,8 +640,25 @@ hildon_desktop_popup_window_composited_button_release (GtkWidget *widget,
 
   /* Event outside of popup or in button area, close in clean way */
   if (!in_panes_area || in_window_area)
+  {	  
     hildon_desktop_popup_window_popdown (popup);
+#ifdef HAVE_XTEST
+    if (popup->priv->attached_widget)
+    {	    
+      gtk_widget_get_pointer (popup->priv->attached_widget, &x, &y);
 
+      w = popup->priv->attached_widget->allocation.width;
+      h = popup->priv->attached_widget->allocation.height;
+      
+      if ((x < 0) || (x > w) || (y < 0) || (y > h))
+      {	      
+        hildon_desktop_popup_menu_fake_button_event (event, TRUE);	      
+        hildon_desktop_popup_menu_fake_button_event (event, FALSE);
+      }
+    }
+#endif
+  }
+  
   return TRUE;
 }
 
@@ -673,8 +699,24 @@ hildon_desktop_popup_window_button_release_event (GtkWidget *widget,
 
   /* Event outside of popup or in button area, close in clean way */
   if (!in_panes_area || in_window_area)
+  {	  
     hildon_desktop_popup_window_popdown (popup);
+#ifdef HAVE_XTEST
+    if (popup->priv->attached_widget)
+    {	    
+      gtk_widget_get_pointer (popup->priv->attached_widget, &x, &y);
 
+      w = popup->priv->attached_widget->allocation.width;
+      h = popup->priv->attached_widget->allocation.height;
+      
+      if ((x < 0) || (x > w) || (y < 0) || (y > h))
+      {	      
+        hildon_desktop_popup_menu_fake_button_event (event, TRUE);	      
+        hildon_desktop_popup_menu_fake_button_event (event, FALSE);
+      }
+    }
+#endif
+  }
   return TRUE;
 }
 
@@ -801,6 +843,13 @@ hildon_desktop_popup_window_set_property (GObject *object,
   }
 }
 
+#ifdef HAVE_XTEST
+static void 
+hildon_desktop_popup_menu_fake_button_event (GdkEventButton *event, gboolean press)
+{
+  XTestFakeButtonEvent (GDK_DISPLAY (), event->button, press, CurrentTime);
+}	
+#endif
 static GdkWindow *
 popup_window_grab_transfer_window_get (HildonDesktopPopupWindow *popup)
 {
@@ -947,6 +996,14 @@ hildon_desktop_popup_window_jump_to_pane (HildonDesktopPopupWindow *popup, gint
   }
 }	
 
+void 
+hildon_desktop_popup_window_attach_widget (HildonDesktopPopupWindow *popup, GtkWidget *widget)
+{
+  g_return_if_fail (widget && GTK_IS_WIDGET (widget));
+
+  popup->priv->attached_widget = widget;
+}	
+
 void 
 hildon_desktop_popup_window_popup (HildonDesktopPopupWindow *popup,
 				   HDPopupWindowPositionFunc func,
