@@ -30,11 +30,6 @@
 #include <hildon/hildon-help.h>
 #endif
 
-#ifdef HAVE_X_COMPOSITE
-#include <X11/extensions/Xrender.h>
-#include <X11/extensions/Xcomposite.h>
-#endif
-
 #ifdef HAVE_LIBOSSO
 #include <libosso.h>
 #endif
@@ -478,7 +473,7 @@ hd_home_window_constructor (GType                   gtype,
 
   if (priv->layout_mode_sucks)
     {
-      g_signal_connect_swapped (area, "applet-change-end",
+      g_signal_connect_swapped (area, "layout-changed",
                                 G_CALLBACK (hd_home_window_area_changed),
                                 window);
     }
@@ -711,12 +706,6 @@ hd_home_window_realize (GtkWidget *widget)
 
   GTK_WIDGET_CLASS (hd_home_window_parent_class)->realize (widget);
 
-#ifdef HAVE_X_COMPOSITE
-  XCompositeRedirectWindow (GDK_DISPLAY (),
-                            GDK_WINDOW_XID (widget->window),
-                            CompositeRedirectAutomatic);
-#endif
-
   wm = hd_wm_get_singleton ();
   g_object_set (wm,
                 "desktop-window", (gint)GDK_WINDOW_XID (widget->window),
@@ -728,11 +717,6 @@ static void
 hd_home_window_unrealize (GtkWidget *widget)
 {
   HDWM *wm;
-#ifdef HAVE_X_COMPOSITE
-  XCompositeUnredirectWindow (GDK_DISPLAY (),
-                              GDK_WINDOW_XID (widget->window),
-                              CompositeRedirectAutomatic);
-#endif
 
   wm = hd_wm_get_singleton ();
   g_object_set (wm,
@@ -858,13 +842,17 @@ static gboolean
 hd_home_window_key_press_event (GtkWidget *widget,
                                 GdkEventKey *event)
 {
+  HDHomeWindowPrivate  *priv;
+  priv = HD_HOME_WINDOW_GET_PRIVATE (widget);
+
   switch (event->keyval)
     {
       case HILDON_HARDKEY_MENU:
           hildon_home_window_toggle_menu (HILDON_HOME_WINDOW (widget));
           break;
       case HILDON_HARDKEY_ESC:
-          hd_home_window_cancel_layout (HD_HOME_WINDOW (widget));
+          if (!priv->layout_mode_sucks)
+            hd_home_window_cancel_layout (HD_HOME_WINDOW (widget));
 
           break;
       default:
@@ -1358,6 +1346,7 @@ hd_home_window_area_changed (HDHomeWindow *window)
   HDHomeWindowPrivate  *priv;
 
   priv = HD_HOME_WINDOW_GET_PRIVATE (window);
+  g_debug ("layout changed, saving");
   if (!priv->save_area_timeout)
     priv->save_area_timeout = g_timeout_add (SAVE_TIMEOUT,
                                              (GSourceFunc)
@@ -1396,7 +1385,8 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
       /* if no applets are set we disable the layout_mode item
        * and the settings item.
        */
-      gtk_widget_set_sensitive (priv->layout_mode_item, FALSE);
+      if (!priv->layout_mode_sucks)
+        gtk_widget_set_sensitive (priv->layout_mode_item, FALSE);
       gtk_widget_set_sensitive (priv->settings_item, FALSE);
       return;
     }
@@ -1420,7 +1410,8 @@ hd_home_window_ensure_menu_status (HDHomeWindow *window)
         }
     }
 
-  gtk_widget_set_sensitive (priv->layout_mode_item, TRUE);
+  if (!priv->layout_mode_sucks)
+    gtk_widget_set_sensitive (priv->layout_mode_item, TRUE);
   gtk_widget_set_sensitive (priv->settings_item, settings_item_active);
   g_list_free (items);
 }
