@@ -408,6 +408,10 @@ hn_app_button_create_menu (HNAppButton *app_button)
 
   menu = gtk_menu_new ();
 
+  gtk_menu_attach_to_widget (GTK_MENU (menu), 
+		  	     GTK_WIDGET (app_button),
+			     NULL);
+
   children = hd_entry_info_get_children (info);
   for (l = children; l != NULL; l = l->next)
     {
